/*
 nruler - transparent screen ruler for .NET platform
 Copyright (C) 2007 siliconforks.com

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

public class NRuler: Form {
  private Form help = new Help();

  private int length = 600;

  private const int breadth = 80;

  private Orientation orientation = Orientation.Horizontal;

  private const double TransparentOpacity = 0.65f; 

  private bool isMoving = false;
  private bool isResizing = false;

  private int previousX = -1;
  private int previousY = -1;
  private int previousLength = -1;

  private Font privateFont;

  MenuItem horizontalMenuItem;
  MenuItem verticalMenuItem;
  MenuItem opaqueMenuItem;
  MenuItem alwaysOnTopMenuItem;

  public NRuler() {
    Text = "NRuler";

    // Under Mono, changing the default size causes the entire form to resize???!?
    // Font = new Font(FontFamily.GenericSansSerif, 9, GraphicsUnit.Pixel);
    privateFont = new Font(FontFamily.GenericSansSerif, 9, GraphicsUnit.Pixel);

    // double buffering
    SetStyle(ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint | ControlStyles.DoubleBuffer, true);
    
    // no title bar
    FormBorderStyle = FormBorderStyle.None;

    // set the size
    resize();

    // transparent
    Opacity = TransparentOpacity;

    // right-click menu
    ContextMenu = new ContextMenu();
    MenuItem helpMenuItem = new MenuItem("H&elp");
//    helpMenuItem.Shortcut = Shortcut.F1;
    helpMenuItem.Click += new EventHandler(helpMenuItem_Click);
    ContextMenu.MenuItems.Add(helpMenuItem);
    horizontalMenuItem = new MenuItem("&Horizontal");
//    horizontalMenuItem.Shortcut = Shortcut.CtrlH;
    horizontalMenuItem.Click += new EventHandler(horizontalMenuItem_Click);
    horizontalMenuItem.Enabled = false;
    ContextMenu.MenuItems.Add(horizontalMenuItem);
    verticalMenuItem = new MenuItem("&Vertical");
//    verticalMenuItem.Shortcut = Shortcut.CtrlV;
    verticalMenuItem.Click += new EventHandler(verticalMenuItem_Click);
    ContextMenu.MenuItems.Add(verticalMenuItem);
    opaqueMenuItem = new MenuItem("&Opaque");
    opaqueMenuItem.Click += new EventHandler(opaqueMenuItem_Click);
    ContextMenu.MenuItems.Add(opaqueMenuItem);
    alwaysOnTopMenuItem = new MenuItem("Always on &Top");
//    alwaysOnTopMenuItem.Shortcut = Shortcut.CtrlT;
    alwaysOnTopMenuItem.Click += new EventHandler(alwaysOnTopMenuItem_Click);
    ContextMenu.MenuItems.Add(alwaysOnTopMenuItem);
    MenuItem exitMenuItem = new MenuItem("E&xit");
//    exitMenuItem.Shortcut = Shortcut.CtrlQ;
    exitMenuItem.Click += new EventHandler(exitMenuItem_Click);
    ContextMenu.MenuItems.Add(exitMenuItem);
  }

  private void resize() {
    Invalidate();  // mono needs this
    if (length < 200) {
      length = 200;
    }
    if (orientation == Orientation.Horizontal) {
      ClientSize = new Size(length, breadth);
    }
    else {
      ClientSize = new Size(breadth, length);
    }
  }

  private bool isOverHandle(MouseEventArgs e) {
    int l;
    if (orientation == Orientation.Horizontal) {
      l = e.X;
    }
    else {
      l = e.Y;
    }
    return l >= length - 10;
  }

  private void toggleHelp() {
    help.Show();
  }

  private void horizontal() {
    orientation = Orientation.Horizontal;
    horizontalMenuItem.Enabled = false;
    verticalMenuItem.Enabled = true;
    resize();
  }

  private void vertical() {
    orientation = Orientation.Vertical;
    horizontalMenuItem.Enabled = true;
    verticalMenuItem.Enabled = false;
    resize();
  }

  private void toggleOpaque() {
    if (Opacity == 1.0) {
      Opacity = TransparentOpacity;
      opaqueMenuItem.Checked = false;
    }
    else {
      Opacity = 1.0;
      opaqueMenuItem.Checked = true;
    }
  }

  private void toggleAlwaysOnTop() {
    TopMost = ! TopMost;
    alwaysOnTopMenuItem.Checked = TopMost;
  }

  private void quit() {
    Application.Exit();
  }

  private void helpMenuItem_Click(object o, EventArgs e) {
    toggleHelp();
  }

  private void horizontalMenuItem_Click(object o, EventArgs e) {
    horizontal();
  }

  private void verticalMenuItem_Click(object o, EventArgs e) {
    vertical();
  }

  private void opaqueMenuItem_Click(object o, EventArgs e) {
    toggleOpaque();
  }

  private void alwaysOnTopMenuItem_Click(object o, EventArgs e) {
    toggleAlwaysOnTop();
  }

  private void exitMenuItem_Click(object o, EventArgs e) {
    quit();
  }

  protected override void OnMouseDown(MouseEventArgs e) {
    base.OnMouseDown(e);
    if (e.Button != MouseButtons.Left) {
      return;
    }
    if (isOverHandle(e)) {
      isResizing = true;
      previousX = e.X;
      previousY = e.Y;
      previousLength = length;
    }
    else {
      isMoving = true;
    }
  }

  protected override void OnMouseMove(MouseEventArgs e) {
    base.OnMouseMove(e);
    if (isMoving) {
      int currentX = e.X + DesktopLocation.X;
      int currentY = e.Y + DesktopLocation.Y;
      if (previousX >= 0 || previousY >= 0) {
        int deltaX = currentX - previousX;
        int deltaY = currentY - previousY;
        DesktopLocation = new Point(DesktopLocation.X + deltaX, DesktopLocation.Y + deltaY);
      }
      previousX = currentX;
      previousY = currentY;
    }
    else if (isResizing) {
      int delta;
      if (orientation == Orientation.Horizontal) {
        delta = e.X - previousX;
      }
      else {
        delta = e.Y - previousY;
      }
      length = previousLength + delta;
      resize();
    }
    else {
      if (isOverHandle(e)) {
        if (orientation == Orientation.Horizontal) {
          Cursor = Cursors.SizeWE;
        }
        else {
          Cursor = Cursors.SizeNS;
        }
      }
      else {
        Cursor = Cursors.SizeAll;
      }
    }
  }

  protected override void OnMouseUp(MouseEventArgs e) {
    base.OnMouseUp(e);
    if (e.Button == MouseButtons.Left) {
      isMoving = false;
      isResizing = false;
      previousX = -1;
      previousY = -1;
    }
  }

  protected override void OnKeyPress(KeyPressEventArgs e) {
    base.OnKeyPress(e);
    if (isMoving || isResizing) {
      return;
    }
    switch(e.KeyChar) {
    case 'Q':
    case 'q':
      quit();
      break;
    case 'H':
    case 'h':
      horizontal();
      break;
    case 'V':
    case 'v':
      vertical();
      break;
    case 'O':
    case 'o':
      toggleOpaque();
      break;
    case 'T':
    case 't':
      toggleAlwaysOnTop();
      break;
    default:
      break;
    }
  }

  protected override void OnKeyDown(KeyEventArgs e) {
    base.OnKeyDown(e);
    if (isMoving || isResizing) {
      return;
    }
    int delta = e.Control? 10: 1;
    switch (e.KeyCode) {
    case Keys.Down:
      SetDesktopLocation(DesktopLocation.X, DesktopLocation.Y + delta);
      break;
    case Keys.Up:
      SetDesktopLocation(DesktopLocation.X, DesktopLocation.Y - delta);
      break;
    case Keys.Left:
      SetDesktopLocation(DesktopLocation.X - delta, DesktopLocation.Y);
      break;
    case Keys.Right:
      SetDesktopLocation(DesktopLocation.X + delta, DesktopLocation.Y);
      break;
    case Keys.Oemplus:
      length += delta;
      resize();
      break;
    case Keys.OemMinus:
      length -= delta;
      resize();
      break;
    case Keys.F1:  // FIXME: this should be handled with a menu shortcut
    case Keys.OemQuestion:
      toggleHelp();
      break;
    }
  }

  protected override void OnActivated(EventArgs e) {
    base.OnActivated(e);
    BackColor = Color.LightBlue;
  }

  protected override void OnDeactivate(EventArgs e) {
    base.OnDeactivate(e);
    BackColor = DefaultBackColor;
  }

  protected override void OnPaint(PaintEventArgs e) {
    base.OnPaint(e);
    Pen pen = new Pen(Color.Black);
    Brush brush = new SolidBrush(Color.Red);
    if (orientation == Orientation.Horizontal) {
      e.Graphics.FillRectangle(brush, length - 10, 0, 10, ClientRectangle.Height);
    }
    else {
      e.Graphics.FillRectangle(brush, 0, length - 10, ClientRectangle.Width, 10);
    }
//    e.Graphics.DrawRectangle(pen, 0, 0, ClientRectangle.Width - 1, ClientRectangle.Height - 1);
    if (orientation == Orientation.Horizontal) {
      for (int i = 0; i < length; i++) {
        if (i % 10 == 0) {
          e.Graphics.DrawLine(pen, i, 0, i, 10);
          e.Graphics.DrawLine(pen, i, breadth - 10, i, breadth);
        }
        else if (i % 2 == 0) {
          e.Graphics.DrawLine(pen, i, 0, i, 5);
          e.Graphics.DrawLine(pen, i, breadth - 5, i, breadth);
        }
        // draw the number
        if (i % 20 == 0) {
          String number = i.ToString();
          SizeF size = e.Graphics.MeasureString(number, Font);
          e.Graphics.DrawString(number, privateFont, pen.Brush, i - size.Width / 2, 10);
          e.Graphics.DrawString(number, privateFont, pen.Brush, i - size.Width / 2, breadth - 10 - size.Height);
        }
      }
    }
    else {
      for (int i = 0; i < length; i++) {
        if (i % 10 == 0) {
          e.Graphics.DrawLine(pen, 0, i, 10, i);
          e.Graphics.DrawLine(pen, breadth - 10, i, breadth, i);
        }
        else if (i % 2 == 0) {
          e.Graphics.DrawLine(pen, 0, i, 5, i);
          e.Graphics.DrawLine(pen, breadth - 5, i, breadth, i);
        }
        // draw the number
        if (i % 20 == 0) {
          String number = i.ToString();
          SizeF size = e.Graphics.MeasureString(number, Font);
          e.Graphics.DrawString(number, privateFont, pen.Brush, 11, i - size.Height / 2);
          e.Graphics.DrawString(number, privateFont, pen.Brush, breadth - 11 - size.Width, i - size.Height / 2);
        }
      }
    }
  }

  public static int Main(string[] args) {
    Application.Run(new NRuler());
    return 0;
  }
}

public class Help: Form {
  public Help() {
    Text = "NRuler Help";

    Label label = new Label();
    if (Type.GetType("Mono.Runtime") != null || Environment.Version.Major == 1) {
      // workaround for Mono and Microsoft .NET Framework CLR 1.1
      label.Size = new Size(400, 200);
    }
    else {
      label.AutoSize = true;
    }
    label.Text = "NRuler version 0.1\n" +
                 "http://siliconforks.com/nruler/\n\n" + 
                 "h: make horizontal\n" +
                 "v: make vertical\n" +
                 "o: make opaque\n" +
                 "t: always on top\n" +
                 "+, -: resize 1 px (Ctrl: 10 px)\n" +
                 "left, right, up, down: move 1 px (Ctrl: 10 px)\n" +
                 "F1, ?: this help\n" +
                 "q: quit";
    label.Font = new Font(FontFamily.GenericMonospace, 12, GraphicsUnit.Pixel);
    Controls.Add(label);

    Button button = new Button();
    button.Text = "Close";
    button.Anchor = AnchorStyles.Bottom | AnchorStyles.Right;
    button.Click += new EventHandler(button_Click);
    Controls.Add(button);

    label.Location = new Point(0, 0);
    button.Location = new Point(ClientSize.Width - 10 - button.Width, ClientSize.Height - 10 - button.Height);
    ClientSize = new Size(label.Width, label.Height + 10 + button.Height + 10);
  }

  private void button_Click(object o, EventArgs e) {
    Hide();
  }

  protected override void OnClosing(CancelEventArgs e) {
    base.OnClosing(e);
    e.Cancel = true;
    Hide();
  }
}
